#!/usr/bin/perl -w

# couriergraph -- a postfix statistics rrdtool frontend
# copyright (c) 2002 Ralf Hildebrandt <ralf.hildebrandt@charite.de>
# based upon mailgraph, which is
# copyright (c) 2000, 2001, 2002 David Schweikert <dws@ee.ethz.ch>
# released under the GNU General Public License

use RRDs;
use POSIX qw(uname);

my $VERSION = 0.21;

my $host = (POSIX::uname())[1];
my $scriptname = 'couriergraph.cgi';
my $xpoints = 540;
my $points_per_sample = 3;
my $ypoints = 150;
my $ypoints_err = 80;
my $rrd = '/var/lib/couriergraph/couriergraph.rrd';	# path to where the RRD database is
my $tmp_dir = '/var/cache/couriergraph'; 	# temporary directory where to store the images

my @graphs = (
	{ title => 'Day Graph',   seconds => 3600*24,        },
	{ title => 'Week Graph',  seconds => 3600*24*7,      },
	{ title => 'Month Graph', seconds => 3600*24*31,     },
	{ title => 'Year Graph',  seconds => 3600*24*365, },
);

sub graph($$$)
{
	my $range = shift;
	my $file = shift;
	my $title = shift;
	my $step = $range*$points_per_sample/$xpoints;

	my ($graphret,$xs,$ys) = RRDs::graph($file,
		'--imgformat', 'PNG',
		'--width', $xpoints,
		'--height', $ypoints,
		'--start', "-$range",
		'--end', "-".int($range*0.01),
		'--vertical-label', 'logins/min',
		'--title', $title,
		'--lazy',
		"DEF:pop3d_login=$rrd:pop3d_login:AVERAGE",
		"DEF:mpop3d_login=$rrd:pop3d_login:MAX",
		"DEF:imapd_login=$rrd:imapd_login:AVERAGE",
		"DEF:mimapd_login=$rrd:imapd_login:MAX",
		"CDEF:rpop3d_login=pop3d_login,60,*",
		"CDEF:vpop3d_login=pop3d_login,UN,0,pop3d_login,IF,$range,*",
		"CDEF:rmpop3d_login=mpop3d_login,60,*",
		"CDEF:rimapd_login=imapd_login,60,*",
		"CDEF:vimapd_login=imapd_login,UN,0,imapd_login,IF,$range,*",
		"CDEF:rmimapd_login=mimapd_login,60,*",
		'LINE2:rpop3d_login#BB0000:pop3',
		'GPRINT:vpop3d_login:AVERAGE:total\: %.0lf logins',
		'GPRINT:rmpop3d_login:MAX:max\: %.0lf logins/min\l',
		'LINE2:rimapd_login#009900:imap',
		'GPRINT:vimapd_login:AVERAGE:total\: %.0lf logins',
		'GPRINT:rmimapd_login:MAX:max\: %.0lf logins/min\l',
		'HRULE:0#000000',
		'COMMENT:\s',
		'COMMENT:Graph Created on '.localtime(time).'\r',
	);
	my $ERR=RRDs::error;
	die "ERROR: $ERR\n" if $ERR;
}

sub graph_err($$$)
{
	my $range = shift;
	my $file = shift;
	my $title = shift;
	my $step = $range*$points_per_sample/$xpoints;

	my ($graphret,$xs,$ys) = RRDs::graph($file,
		'--imgformat', 'PNG',
		'--width', $xpoints,
		'--height', $ypoints,
		'--start', "-$range",
		'--end', "-".int($range*0.01),
		'--vertical-label', 'logins/hour',
		'--title', "$title (SSL)",
		'--lazy',
		"DEF:pop3d_ssl_login=$rrd:pop3d_ssl_login:AVERAGE",
		"DEF:mpop3d_ssl_login=$rrd:pop3d_ssl_login:MAX",
		"DEF:imapd_ssl_login=$rrd:imapd_ssl_login:AVERAGE",
		"DEF:mimapd_ssl_login=$rrd:imapd_ssl_login:MAX",
		"CDEF:rpop3d_ssl_login=pop3d_ssl_login,3600,*",
		"CDEF:vpop3d_ssl_login=pop3d_ssl_login,UN,0,pop3d_ssl_login,IF,$range,*",
		"CDEF:rmpop3d_ssl_login=mpop3d_ssl_login,3600,*",
		"CDEF:rimapd_ssl_login=imapd_ssl_login,3600,*",
		"CDEF:rmimapd_ssl_login=mimapd_ssl_login,3600,*",
		"CDEF:vimapd_ssl_login=imapd_ssl_login,UN,0,imapd_ssl_login,IF,$range,*",
		'LINE2:rpop3d_ssl_login#000000:pop3',
		'GPRINT:vpop3d_ssl_login:AVERAGE:total\: %.0lf logins',
		'GPRINT:rmpop3d_ssl_login:MAX:max\: %.0lf logins/hour\l',
		'LINE2:rimapd_ssl_login#000099:imap',
		'GPRINT:vimapd_ssl_login:AVERAGE:total\: %.0lf logins',
		'GPRINT:rmimapd_ssl_login:MAX:max\: %.0lf logins/hour\l',
		'HRULE:0#000000',
		'COMMENT:\s',
		'COMMENT:Graph Created on '.localtime(time).'\r',
	);
	my $ERR=RRDs::error;
	die "ERROR: $ERR\n" if $ERR;
}

sub print_html()
{
	print "Content-Type: text/html\n\n";

	print <<HEADER;
<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.0 Transitional//EN" "http://www.w3.org/TR/REC-html40/loose.dtd">
<HTML>
<HEAD>
<TITLE>Courier Login Statistics for $host</TITLE>
</HEAD>
<BODY BGCOLOR="#FFFFFF">
HEADER

	print "<H1>Courier Login Statistics for $host</H1>\n";
	for my $n (0..$#graphs) {
		print "<H2>$graphs[$n]{title}</H2>\n";
		print "<P><IMG BORDER=\"0\" SRC=\"/cgi-bin/$scriptname/couriergraph_${n}.png\" ALT=\"couriergraph\">\n";
		print "<P><IMG BORDER=\"0\" SRC=\"/cgi-bin/$scriptname/couriergraph_${n}_err.png\" ALT=\"couriergraph\">\n";
	}

	print <<FOOTER;
<table border="0" width="500"><tr><td align="left">
<A href="http://www.stahl.bau.tu-bs.de/~hildeb/postfix/couriergraph">Couriergraph</A> $VERSION
by <A href="http://www.stahl.bau.tu-bs.de/~hildeb/">Ralf Hildebrandt</A>, based upon
<A href="http://people.ee.ethz.ch/~dws/software/mailgraph">Mailgraph</A> $VERSION
by <A href="http://people.ee.ethz.ch/~dws/">David Schweikert</A></td>
<td ALIGN="right">
<a HREF="http://people.ee.ethz.ch/~oetiker/webtools/rrdtool/"><img border="0" src="http://people.ethz.ch/~oetiker/webtools/rrdtool/.pics/rrdtool.gif" alt="" width="120" height="34"></a>
</td></tr></table>
</BODY></HTML>
FOOTER
}

sub send_image($)
{
	my $file = shift;
	-r $file or do {
		print "Content-type: text/plain\n\nERROR: can't find $file\n";
		exit 1;
	};

	print "Content-Type: image/png\n";
	print "Content-Length: ".((stat($file))[7])."\n";
	print "\n";
	open(IMG, $file) or die;
	my $data;
	print $data while read IMG, $data, 1024;
}

sub main()
{
	if($ENV{PATH_INFO}) {
		my $uri = $ENV{REQUEST_URI};
		$uri =~ s/\/[^\/]+$//;
		$uri =~ s/\//,/g;
		$uri =~ s/(\~|\%7E)/tilde,/g;
		mkdir $tmp_dir, 0777 unless -d $tmp_dir;
		mkdir "$tmp_dir/$uri", 0777 unless -d "$tmp_dir/$uri";
		my $file = "$tmp_dir/$uri$ENV{PATH_INFO}";
		if($ENV{PATH_INFO} =~ /^\/couriergraph_(\d+)\.png$/) {
			graph($graphs[$1]{seconds}, $file, $graphs[$1]{title});
		}
		elsif($ENV{PATH_INFO} =~ /^\/couriergraph_(\d+)_err\.png$/) {
			graph_err($graphs[$1]{seconds}, $file, $graphs[$1]{title});
		}
		else {
			print "Content-Type: text/plain\n\nERROR: unknown image $ENV{PATH_INFO}\n";
			exit 1;
		}
		send_image($file);
	}
	else {
		print_html;
	}
}

main;
